==> scanbot_sdk-2.4.1.tar.gz: android/src/main/kotlin/io/scanbot/sdk/flutter/PageFileStoragePlugin.kt <==
/*
    Scanbot SDK Flutter Plugin

    Copyright (c) 2019 doo GmbH. All rights reserved.
 */
package io.scanbot.sdk.flutter

import android.content.Context
import android.net.Uri
import com.google.gson.FieldNamingPolicy
import com.google.gson.GsonBuilder
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel.Result
import io.scanbot.sdk.ScanbotSDK
import io.scanbot.sdk.flutter.utils.StorageUtils
import io.scanbot.sdk.process.ImageFilterType
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.withContext
import java.io.File

class PageFileStoragePlugin(
        val context: Context,
        coroutineResultHandler: CoroutineResultHandler,
        scanbotSDK: ScanbotSDK
) : ISdkSubplugin,
        ICoroutineActivityResultHandler by coroutineResultHandler {


    private val gson = GsonBuilder()
            .setFieldNamingPolicy(FieldNamingPolicy.IDENTITY)
            .create()
    private val pageFileStorage = scanbotSDK.pageFileStorage()
    private val pageProcessor = scanbotSDK.pageProcessor()
    private val fileIOProcessor = scanbotSDK.fileIOProcessor()

    override suspend fun onMethodCall(call: MethodCall, result: Result): Boolean {

        when (call.method) {
            "getFilteredDocumentPreviewUri" -> getFilteredDocumentPreviewUri(call, result)
            "getDecryptedDataFromFile" -> getDecryptedDataFromFile(call, result)
            "deletePage" -> deletePage(call, result)
            "cleanupStorage" -> cleanupStorage(context, call, result)
            else -> return false
        }
        return true
    }


    private suspend fun getFilteredDocumentPreviewUri(
            call: MethodCall,
            methodResult: Result
    ) {
        withContext(Dispatchers.Default) {
            try {
                val pageJson = call.argument<PageJson>("page") ?: PageJson()
                val page = pageJson.toPage(gson)

                val imageFilterType = call.argument<String>("filter") ?: ""
                val filter = ImageFilterType.valueOf(imageFilterType)

                val previewFileUri = pageFileStorage.getFilteredPreviewImageURI(page.pageId, filter)
                if (!File(previewFileUri.path).exists()) {
                    pageProcessor.generateFilteredPreview(page, filter)
                }

                withContext(Dispatchers.Main) { methodResult.success(previewFileUri.toString()) }
            } catch (e: Throwable) {
                withContext(Dispatchers.Main) {
                    methodResult.error(
                            "RuntimeException",
                            "Could not process page data",
                            null
                    )
                }
            }
        }
    }


    private suspend fun getDecryptedDataFromFile(
            call: MethodCall,
            methodResult: Result
    ) {
        withContext(Dispatchers.Default) {
            try {
                val imageUriString = call.argument<String?>("fileUri")
                if (imageUriString.isNullOrEmpty()) {
                    withContext(Dispatchers.Main) {
                        methodResult.error(
                                "SdkParamException",
                                "imageFileUri is null or empty",
                                null
                        )
                    }
                    return@withContext
                }

                val file = File(Uri.parse(imageUriString).path!!)
                if (!file.exists()) {
                    withContext(Dispatchers.Main) {
                        methodResult.error(
                                "FileAccessException",
                                "\"File not found",
                                null
                        )
                    }
                    return@withContext
                }
                val decodedFile = fileIOProcessor.read(file)
                // In case image file encryption is enabled, the corresponding FileIOProcessor
                // implementation will decrypt image data under the hood:
                withContext(Dispatchers.Main) { methodResult.success(decodedFile) }
            } catch (e: Throwable) {
                withContext(Dispatchers.Main) {
                    methodResult.error(
                            "RuntimeException",
                           "error decoding image from file",
                            null
                    )
                }
            }
        }
    }


    private suspend fun deletePage(
            call: MethodCall,
            methodResult: Result
    ) {
        withContext(Dispatchers.Default) {
            try {
                val pageJson = call.argument<PageJson>("page") ?: PageJson()
                val page = pageJson.toPage(gson)

                val pageId = page.pageId ?: ""
                val deleted = pageFileStorage.remove(pageId)
                if (deleted) {
                    withContext(Dispatchers.Main) { methodResult.success("") }
                } else {
                    withContext(Dispatchers.Main) {
                        methodResult.error(
                                "PageException",
                                "Could not delete page",
                                null
                        )
                    }
                }
            } catch (e: Throwable) {
                withContext(Dispatchers.Main) {
                    methodResult.error(
                            "PageException",
                            "error getting page params",
                            null
                    )
                }
            }
        }
    }


    private suspend fun cleanupStorage(
            context: Context,
            call: MethodCall,
            methodResult: Result
    ) {
        withContext(Dispatchers.Default) {
            try {
                // cleanup the pages storage
                pageFileStorage.removeAll()

                // cleanup the plugin storage folder (generated TIFF, PDF, etc)
                StorageUtils.cleanupPluginStorageDirectory(context)

                withContext(Dispatchers.Main) { methodResult.success("") }
            } catch (e: Throwable) {
                withContext(Dispatchers.Main) {
                    methodResult.error(
                            "RuntimeException",
                            "error while cleaning storage",
                            null
                    )
                }
            }
        }
    }
}

==> scanbot_sdk-2.9.0.tar.gz: android/src/main/kotlin/io/scanbot/sdk/flutter/PageFileStoragePlugin.kt <==
/*
    Scanbot SDK Flutter Plugin

    Copyright (c) 2019 doo GmbH. All rights reserved.
 */
package io.scanbot.sdk.flutter

import android.content.Context
import android.net.Uri
import com.google.gson.FieldNamingPolicy
import com.google.gson.GsonBuilder
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel.Result
import io.scanbot.sdk.ScanbotSDK
import io.scanbot.sdk.flutter.utils.StorageUtils
import io.scanbot.sdk.process.ImageFilterType
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.withContext
import java.io.File

class PageFileStoragePlugin(
        val context: Context,
        coroutineResultHandler: CoroutineResultHandler,
        scanbotSDK: ScanbotSDK
) : ISdkSubplugin,
        ICoroutineActivityResultHandler by coroutineResultHandler {


    private val gson = GsonBuilder()
            .setFieldNamingPolicy(FieldNamingPolicy.IDENTITY)
            .create()
    private val pageFileStorage = scanbotSDK.createPageFileStorage()
    private val pageProcessor = scanbotSDK.createPageProcessor()
    private val fileIOProcessor = scanbotSDK.fileIOProcessor()

    override suspend fun onMethodCall(call: MethodCall, result: Result): Boolean {

        when (call.method) {
            "getFilteredDocumentPreviewUri" -> getFilteredDocumentPreviewUri(call, result)
            "getDecryptedDataFromFile" -> getDecryptedDataFromFile(call, result)
            "deletePage" -> deletePage(call, result)
            "cleanupStorage" -> cleanupStorage(context, call, result)
            else -> return false
        }
        return true
    }


    private suspend fun getFilteredDocumentPreviewUri(
            call: MethodCall,
            methodResult: Result
    ) {
        withContext(Dispatchers.Default) {
            try {
                val pageJson = call.argument<Json>("page") ?: Json()
                val page = pageJson.toPage(gson)

                val imageFilterType = call.argument<String>("filter") ?: ""
                val filter = ImageFilterType.valueOf(imageFilterType)

                val previewFileUri = pageFileStorage.getFilteredPreviewImageURI(page.pageId, filter)
                if (!File(previewFileUri.path).exists()) {
                    pageProcessor.generateFilteredPreview(page, filter)
                }

                withContext(Dispatchers.Main) { methodResult.success(previewFileUri.toString()) }
            } catch (e: Throwable) {
                withContext(Dispatchers.Main) {
                    methodResult.error(
                            "RuntimeException",
                            "Could not process page data",
                            null
                    )
                }
            }
        }
    }


    private suspend fun getDecryptedDataFromFile(
            call: MethodCall,
            methodResult: Result
    ) {
        withContext(Dispatchers.Default) {
            try {
                val imageUriString = call.argument<String?>("fileUri")
                if (imageUriString.isNullOrEmpty()) {
                    withContext(Dispatchers.Main) {
                        methodResult.error(
                                "SdkParamException",
                                "imageFileUri is null or empty",
                                null
                        )
                    }
                    return@withContext
                }

                val file = File(Uri.parse(imageUriString).path!!)
                if (!file.exists()) {
                    withContext(Dispatchers.Main) {
                        methodResult.error(
                                "FileAccessException",
                                "\"File not found",
                                null
                        )
                    }
                    return@withContext
                }
                val decodedFile = fileIOProcessor.read(file)
                // In case image file encryption is enabled, the corresponding FileIOProcessor
                // implementation will decrypt image data under the hood:
                withContext(Dispatchers.Main) { methodResult.success(decodedFile) }
            } catch (e: Throwable) {
                withContext(Dispatchers.Main) {
                    methodResult.error(
                            "RuntimeException",
                           "error decoding image from file",
                            null
                    )
                }
            }
        }
    }


    private suspend fun deletePage(
            call: MethodCall,
            methodResult: Result
    ) {
        withContext(Dispatchers.Default) {
            try {
                val pageJson = call.argument<Json>("page") ?: Json()
                val page = pageJson.toPage(gson)

                val pageId = page.pageId ?: ""
                val deleted = pageFileStorage.remove(pageId)
                if (deleted) {
                    withContext(Dispatchers.Main) { methodResult.success("") }
                } else {
                    withContext(Dispatchers.Main) {
                        methodResult.error(
                                "PageException",
                                "Could not delete page",
                                null
                        )
                    }
                }
            } catch (e: Throwable) {
                withContext(Dispatchers.Main) {
                    methodResult.error(
                            "PageException",
                            "error getting page params",
                            null
                    )
                }
            }
        }
    }


    private suspend fun cleanupStorage(
            context: Context,
            call: MethodCall,
            methodResult: Result
    ) {
        withContext(Dispatchers.Default) {
            try {
                // cleanup the pages storage
                pageFileStorage.removeAll()

                // cleanup the plugin storage folder (generated TIFF, PDF, etc)
                StorageUtils.cleanupPluginStorageDirectory(context)

                withContext(Dispatchers.Main) { methodResult.success("") }
            } catch (e: Throwable) {
                withContext(Dispatchers.Main) {
                    methodResult.error(
                            "RuntimeException",
                            "error while cleaning storage",
                            null
                    )
                }
            }
        }
    }
}
